#if 0  /* autodoc block */

<lalVerbatim file="LoadCV">
$Id: Load.c,v 1.3 2001/08/27 16:40:45 jolien Exp $
</lalVerbatim>

<lalLaTeX>
\subsection{Module \texttt{Load.c}}
\label{ss:Load.c}

The LAL-wrapper interface functions for the \texttt{load} shared object.

\subsubsection*{Description}

This shared object is to test load balancing during a run.

The \verb+filterparams+ input should be a single integer representing the
number of requests to send to the slaves.  Each request makes the slave do
four sleeps.  Between each sleep, the slave may be requested to come-up-for-air
(i.e., syncronize in the wrapperAPI so that load balancing may be done),
and the slave should continue along happily after the load balancing has been
done.  The master comes-up-for-air frequently and services slave requests
(unless it has been directed that load balancing is to be done, in which case
it broadcasts a come-up-for-air message to all the slaves).

\subsubsection*{Algorithm}

\subsubsection*{Uses}

\subsubsection*{Notes}

\vfill{\footnotesize\input{LoadCV}}

</lalLaTeX>

#endif /* autodoc block */

#include <stdio.h>
#include <unistd.h>
#include <lal/LALStdlib.h>
#include <Load.h>

enum { exittag = 1, donetag, rqsttag, srvctag };
const int numperslave = 4;
const int numdefault  = 8; /* number of times numperslave */

NRCSID( LOADC, "$Id: Load.c,v 1.3 2001/08/27 16:40:45 jolien Exp $" ); 

void LALInitSearch(
    LALStatus             *status,
    void                 **searchParams,
    LALInitSearchParams   *initSearchParams
    )
{
  int *ntodo;  /* Array of 3 values.
                *
                * On the master these are:
                *   0: total number of sleeps remaining to do
                *   1: number of requests (of numperslave sleeps) to send
                *   2: original total number of sleeps to do
                *
                * On the slave, only ntodo[0] is used, and this represents
                * the number of sleeps to do in this call to the slave.
                */

  INITSTATUS( status, "LALInitSearch", LOADC );

  ASSERT(    searchParams,  status, LOADH_ENULL, LOADH_MSGENULL );
  ASSERT( ! *searchParams,  status, LOADH_ENNUL, LOADH_MSGENNUL );
  ASSERT( initSearchParams, status, LOADH_ENULL, LOADH_MSGENULL );

  *searchParams = ntodo = LALCalloc( 3, sizeof( *ntodo ) );
  if ( ! ntodo )
  {
    ABORT( status, LOADH_EALOC, LOADH_MSGEALOC );
  }

  /* parse arguments */
  switch ( initSearchParams->argc )
  {
    case 1: /* no arguments, default number of times */
      ntodo[1] = numdefault;
      break;

    case 2: /* number is argument */
      ntodo[1] = atoi( initSearchParams->argv[1] );
      break;

    default:
      ABORT( status, LOADH_EARGS, LOADH_MSGEARGS );
  }
  ntodo[2] = ntodo[1] * numperslave;
  printf( "Number to do: %d\n", ntodo[2] );

  if ( initSearchParams->nodeClass ) /* I am the search master */
  {
    ntodo[0] = ntodo[2];
  }
  else
  {
    ntodo[0] = 0;
  }

  RETURN( status );
}


void
LALConditionData(
    LALStatus             *status,
    LALSearchInput        *inout,
    void                  *searchParams,
    LALMPIParams          *mpiParams
    )
{
  INITSTATUS( status, "LALConditionData", LOADC );

  ASSERT( inout, status, LOADH_ENULL, LOADH_MSGENULL );

  RETURN( status );
}


void
LALApplySearch(
    LALStatus             *status,
    LALSearchOutput       *output,
    LALSearchInput        *input,
    LALApplySearchParams  *params
    )
{
  int  rank;
  int *ntodo;
  int  buf = 1; /* a worthless buffer -- most info comes from the tags */
  MPI_Status mpistat;
  MPI_Comm   comm;

  INITSTATUS( status, "LALApplySearch", LOADC );
  
  ASSERT( output, status, LOADH_ENULL, LOADH_MSGENULL );
  ASSERT( input,  status, LOADH_ENULL, LOADH_MSGENULL );
  ASSERT( params, status, LOADH_ENULL, LOADH_MSGENULL );

  ntodo = params->searchParams;
  ASSERT( ntodo, status, LOADH_ENULL, LOADH_MSGENULL );

  ASSERT( params->mpiParams, status, LOADH_ENULL, LOADH_MSGENULL );
  ASSERT( params->mpiParams->comm, status, LOADH_ENULL, LOADH_MSGENULL );
  comm = *params->mpiParams->comm;

  MPI_Comm_rank( comm, &rank );

  /* there is no output generated */
  output->numOutput = 0;
  output->result    = NULL;

  if ( ! rank ) /* I am the search master */
  {
    int size;
    MPI_Comm_size( comm, &size );

    if ( params->mpiParams->action && params->mpiParams->action->add )
    { /* load balancing to be done */
      int slave;
      printf( "Search Master: telling slaves to come up for air\n" );
      fflush( stdout );
      /* should this be a Bcast ?  an IBcast ? */
      for ( slave = 1; slave < size; ++slave )
      {
        /* tell slave to exit so we can load balance */
        MPI_Send( &buf, 1, MPI_INT, slave, exittag, comm );
      }
      /*
       * If nodes are subtracted, assume that this number of requests
       * must be re-issued (conservative assumption).
       */
      if ( params->mpiParams->action->add < 0 )
      {
        ntodo[1] -= params->mpiParams->action->add;
      }
    }
    else /* wait for message from some slave */
    {
      int doneflag;
      int rqstflag;
      printf( "Search Master: listening... (remaining no. to be "
          "sent / rcvd = %d / %d of %d)\n",
          ntodo[1] * numperslave, ntodo[0], ntodo[2] );
      fflush( stdout );

      /* see if any slave is done */
      do {
        doneflag = 0;
        MPI_Iprobe( MPI_ANY_SOURCE, donetag, comm, &doneflag, &mpistat );
        if ( doneflag ) /* a slave is done */
        {
          int source = mpistat.MPI_SOURCE;
          printf( "Search Master: notified that slave %d is done\n", source );
          fflush( stdout );
          /* receive the stupid request */
          MPI_Recv( &buf, 1, MPI_INT, source, donetag, comm, &mpistat );
          ntodo[0] -= numperslave;
          if ( ntodo[0] <= 0 )
          {
            goto masterexit;
          }
        }
      }
      while ( doneflag );

      /* see if there is a request pending */
      do {
        rqstflag = 0;
        MPI_Iprobe( MPI_ANY_SOURCE, rqsttag, comm, &rqstflag, &mpistat );
        if ( rqstflag ) /* got a request */
        {
          int source = mpistat.MPI_SOURCE;
          int num;
          printf( "Search Master: servicing request from slave %d\n", source );
          fflush( stdout );
          if ( ntodo[1] > 0 && ntodo[0] > 0 )
          {
            num = numperslave; /* the usual number */
            --ntodo[1];        /* one fewer request to service */
            printf( "Search Master: slave %d told to do %d more\n",
                source, numperslave );
            fflush( stdout );
          }
          else
          {
            printf( "Search Master: slave %d told to finish\n", source );
            fflush( stdout );
            num = -1; /* tell slave to finish */
          }
          /* receive the stupid request */
          MPI_Recv( &buf, 1, MPI_INT, source, rqsttag, comm, &mpistat );
          /* and service the request */
          MPI_Send( &num, 1, MPI_INT, source, srvctag, comm );
        }
      }
      while ( rqstflag );

      sleep( 1 ); /* pause */
    }

masterexit:
    output->fracRemaining = (float) ntodo[0] / (float) ntodo[2];
    output->notFinished   = ntodo[0] ? 1 : 0;
    if ( ! output->notFinished )
    {
      int num = -1;
      int slave;
      printf( "Search Master: sending finish signals to remaining slaves\n" );
      for ( slave = 1; slave < size; ++slave )
      {
        MPI_Send( &num, 1, MPI_INT, slave, srvctag, comm );
      }
      printf( "Search Master: finished\n" );
      fflush( stdout );
    }

  }
  else /* I am a search slave */
  {
    if ( ntodo[0] == 0 ) /* if nothing left to do, request more */
    {
      int exitflag = 0;
      /* first check to see if we should exit */
      MPI_Iprobe( 0, exittag, comm, &exitflag, &mpistat );
      if ( exitflag )
      {
        /* get the stupid message and exit */
        MPI_Recv( &buf, 1, MPI_INT, 0, exittag, comm, &mpistat );
        goto slaveexit;
      }
      else
      {
        printf( "Slave %d: requesting more\n", rank );
        fflush( stdout );
        /* send off request for more todo */
        MPI_Send( &buf, 1, MPI_INT, 0, rqsttag, comm );
        /* get the todo */
        MPI_Recv( ntodo, 1, MPI_INT, 0, srvctag, comm, &mpistat );
      }
    }

    if ( ntodo[0] < 0 ) /* slave has been told it is done */
    {
      printf( "Slave %d: finishing\n", rank );
      fflush( stdout );
      output->fracRemaining = 0; /* finished */
      output->notFinished   = 0; /* finished */
    }
    else
    {
      output->fracRemaining = 1; /* unused */
      output->notFinished   = 1; /* not finished */
      while ( ntodo[0] ) /* loop until done or told to exit */
      {
        int exitflag = 0;
        int sleeptime = rank + 1;
        printf( "Slave %d: %d remaining\n", rank, ntodo[0] );
        fflush( stdout );
        /* iprobe and break if told to exit immediately */
        MPI_Iprobe( 0, exittag, comm, &exitflag, &mpistat );
        if ( exitflag )
        {
          /* get the stupid message and exit */
          MPI_Recv( &buf, 1, MPI_INT, 0, exittag, comm, &mpistat );
          goto slaveexit;
        }
        sleep( sleeptime ); /* this is what we do! */
        --ntodo[0];
      }

      /* tell search master I am done */
      MPI_Send( &buf, 1, MPI_INT, 0, donetag, comm );

slaveexit:
      printf( "Slave %d: coming up for air\n", rank );
      fflush( stdout );
    }
  }

  RETURN( status );
}


void
LALFinalizeSearch(
    LALStatus             *status,
    void                 **searchParams
    )
{
  INITSTATUS( status, "LALFinalizeSearch", LOADC );

  ASSERT(  searchParams, status, LOADH_ENULL, LOADH_MSGENULL );
  ASSERT( *searchParams, status, LOADH_ENULL, LOADH_MSGENULL );

  LALFree( *searchParams );
  *searchParams = NULL;

  RETURN( status );
}
