/************************** <lalVerbatim file="StochasticConditionDataCV">
Author: Anderson, W.G., Romano, J.D., and Whelan, J.T.
$Id: StochasticConditionData.c,v 1.46 2002/08/13 21:16:26 whelan Exp $
**************************************************** </lalVerbatim> */

/**************************** <lalLaTeX file="StochasticConditionDataC">
\subsection{Module \texttt{StochasticConditionData.c}}
\label{s:StochasticConditionData.c}

Routine for performing one-time processing in preparation for the
stochastic search.

\subsubsection*{Prototypes}
\input{StochasticConditionDataCP}
\index{\texttt{LALConditionData()}}

\subsection*{Description}

Performs one-time processing (e.g., extraction of input data, construction
of windowing functions, and
calculation of the optimal filter for a stochastic background having 
$\Omega_{\rm gw}(f)=\Omega_0={\rm const}$) in preparation for the 
stochastic search.

\subsection*{Algorithms}
  
\texttt{LALConditionData} does the following:
%
\begin{enumerate}
\item Constructs the windowing functions requested for the two data streams
\item Checks the validity of the input data, and extracts the 
detector geometry, time-series data, power spectral densities, and 
instrument response functions from the multidim input data.
\item Converts 2-sided power spectral densities (produced by the 
\texttt{datacondAPI}) to 1-sided power spectra having the appropriate 
bandwidth and frequency resolution for construction of the optimal 
filter.
\item Coarse-grains the instrument response functions to the
appropriate bandwidth and frequency resolution for construction of
the optimal filter.
\item Casts the extracted input data to types expected by the LAL 
stochastic routines.
\item Calculates the overlap reduction function given the 
detector geometry information.
\item Sets $\Omega_{\rm gw}(f)=\Omega_0 = {\rm const}$ for this
particular search.
\item Calculates inverse calibrated and half-calibrated power spectra 
given the calibrated power spectra and instrument response functions.
\item Calculates the optimal filter for a stochastic background
having $\Omega_{\rm gw}(f)=\Omega_0={\rm const}$ using the 
previously calculated information.
\end{enumerate}

\subsection*{Uses}
 
\begin{itemize}
\item \texttt{LALMalloc}
\item \texttt{LALFree}
\item \texttt{LALCreateVector}
\item \texttt{LALCCreateVector}
\item \texttt{LALSCreateVector}
\item \texttt{LALDestroyVector}
\item \texttt{LALCDestroyVector}
\item \texttt{LALSDestroyVector}
\item \texttt{LALSCoarseGrainFrequencySeries}
\item \texttt{LALCCoarseGrainFrequencySeries}
\item \texttt{LALStochasticOmegaGW}  
\item \texttt{LALStochasticInverseNoise}
\item \texttt{LALStochasticOptimalFilter}
\item \texttt{LALStochasticOptimalFilterNormalization}
\item \texttt{CastToReal4}
\item \texttt{CastToComplex8}
\end{itemize}

\subsection*{Notes}

\begin{itemize}
\item \texttt{LALConditionData} calls two static functions
\texttt{CastToReal4} and \texttt{CastToComplex8}, which are
used to cast time-series data, power spectral densities, 
and instrument response functions to the proper type for the 
LAL stochastic routines.
\item {Since no specific mechanism exists for reading the units string
    from an ILWD into the \texttt{LALUnit} structure, this DSO assumes
    its inputs have the expected units.}  These are
\begin{itemize}
\item Data Streams $h^{\scriptstyle{\rm W}}_{1,2}(t)$:  ADC counts
\item PSDs $P^{\scriptstyle{\rm W}}_{1,2}(f)$:
  (ADC counts)$^2$ / Hz
\item Response functions $\tilde{R}_{1,2}(f)$:
  ADC counts / attostrain = $10^{18}$ counts / strain
\end{itemize}
These are set ``by hand'' in \texttt{LALConditionData()} without
looking at the relevant \texttt{units} fields in the
\texttt{multiDimData}.  This should be changed to parse the
\texttt{units} fields and construct the appropriate \texttt{LALUnit}
structures, or at least check to see if it matches one of a set of
predefined strings.

\end{itemize}

\vfill{\footnotesize\input{StochasticConditionDataCV}}

*************************************************** </lalLaTeX> */

#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <math.h>
#include <lal/LALStdlib.h>
#include <lal/LALConstants.h>
#include <lal/AVFactories.h>
#include <lal/DetectorSite.h>
#include <lal/Units.h>
#include <lal/Window.h>
#include <lal/CoarseGrainFrequencySeries.h>
#include "Stochastic.h"
#include "StochasticConditionData.h"

NRCSID (STOCHASTICCONDITIONDATAC, "$Id: StochasticConditionData.c,v 1.46 2002/08/13 21:16:26 whelan Exp $");

#ifdef STOCHASTIC_SO_DEBUG
static void PrintLALDetector(LALDetector *detector);
#endif

static inline INT4 ctoi (CHAR c);

static void CastToReal4 ( LALStatus *status, REAL4 *output,
			  const multiDimData *input,
			  const UINT4 offset, const UINT4 length );

static void CastToComplex8 ( LALStatus *status, COMPLEX8 *output,
			     const multiDimData *input,
			     const UINT4 offset, const UINT4 length );

/************************ <lalVerbatim file="StochasticConditionDataCP"> */
void LALConditionData (
    LALStatus*          status,
    LALSearchInput*     inout,
    void*               searchParams,
    LALMPIParams          *mpiParams
)
/*********************************************** </lalVerbatim> */
{

  /* Taken from Finn and Lazzarini gr-qc/0104040 */
#define STOCHASTICCONDITIONDATAC_ALLEGROLON ( -( 91.0L + ( 10.0L + (43.766L/60.0L) ) / 60.0L) * LAL_PI_180 )
#define STOCHASTICCONDITIONDATAC_ALLEGROLAT ( 30.0L + ( 24.0L + (45.110L/60.0L) ) / 60.0L * LAL_PI_180  )
#define STOCHASTICCONDITIONDATAC_ALLEGROAZ ( -40.0L * LAL_PI_180 )

    INT4                            i;

    UINT4                           detInd;
    INT4                            spec1Index      = -1;
    INT4                            spec2Index      = -1;
    INT4                            resp1Index      = -1;
    INT4                            resp2Index      = -1;
    INT4                            data1Index      = -1;
    INT4                            data2Index      = -1;

    REAL4FrequencySeries           *spec1, *spec2;
    REAL4FrequencySeries            spec1Raw, spec2Raw;
    REAL4Vector                    *spec1TwoSidedData, *spec2TwoSidedData;
    COMPLEX8FrequencySeries        *resp1, *resp2;
    COMPLEX8FrequencySeries         resp1Raw, resp2Raw;
    COMPLEX8Vector                 *resp1TwoSidedData, *resp2TwoSidedData;

    detGeom                        *geometry;

    INT4                            offset;

    FrequencySamplingParams         freqParams;
    
    LALWindowParams                 windowParams;
    REAL4Vector                    *window;
    UINT4                           halfWin;
    REAL4                          *sPtr;
    REAL4                          *sStopPtr;

    REAL4FrequencySeries            overlap;
    LALFrDetector                   frDetector;
    LALDetectorPair                 detectorPair;
    OverlapReductionFunctionParameters overlapParams;

    REAL4FrequencySeries            omegaGW;
    StochasticOmegaGWParameters     omegaGWParams;

    REAL4FrequencySeries            calibratedInvNoise1;
    REAL4FrequencySeries            calibratedInvNoise2;
    COMPLEX8FrequencySeries         halfCalibratedInvNoise1;
    COMPLEX8FrequencySeries         halfCalibratedInvNoise2;
    
    StochasticInverseNoiseInput     invNoiseInput1;
    StochasticInverseNoiseInput     invNoiseInput2;
    StochasticInverseNoiseOutput    invNoiseOutput1;
    StochasticInverseNoiseOutput    invNoiseOutput2;
    
    StochasticOptimalFilterInput    optimalInput;

    StochasticOptimalFilterNormalizationInput       optNormInput;
    StochasticOptimalFilterNormalizationParameters  optNormParams;
    StochasticOptimalFilterNormalizationOutput      optNormOutput;

    REAL4WithUnits                  lambda;
    LALUnitPair                     unitPair;
    LALUnit                         tmpUnit;
    RAT4                            power;
    
    StochasticRealDataSegmentVector     *sDataSegVec      = NULL;
    StochasticRealDataSegment           *sDummySegment    = NULL;
    StochasticComplexDataSegmentVector  *cDataSegVec      = NULL;
    StochasticComplexDataSegment        *cDummySegment    = NULL;
    StochasticSearchParams              *params           = NULL;

    INT8                            epochNanoSeconds;

    INITSTATUS (status, "LALConditionData", STOCHASTICCONDITIONDATAC);
    ATTATCHSTATUSPTR (status);

    /* 
     *
     * check and cast searchParams 
     *
     */
    
    ASSERT(searchParams,status,STOCHASTICCONDITIONDATAH_ENULL, 
              STOCHASTICCONDITIONDATAH_MSGENULL);
    
    params = (StochasticSearchParams *) searchParams;

    /* 
     *
     * make sure conditioning and analysis of data is done on search master 
     *
     */
    
    if (!(params->searchMaster))
    {
        ABORT(status, 
              STOCHASTICCONDITIONDATAH_EMSTR, 
              STOCHASTICCONDITIONDATAH_MSGEMSTR);
    }

    /*
     *
     *  check the input 
     *
     */

    /* check that inout data exists */
    ASSERT (inout, 
            status,
            STOCHASTICCONDITIONDATAH_ENULL, 
            STOCHASTICCONDITIONDATAH_MSGENULL);
            
    /* check that number of sequences is correct */ 
    /* :NOTE: once the detector data are included this will be 8 */
    if (inout->numberSequences != 6)
    {
        ABORT(status, 
              STOCHASTICCONDITIONDATAH_ENSEQ, 
              STOCHASTICCONDITIONDATAH_MSGENSEQ);
    }
       
    /* check that multiDimData sequence data exists */
    if (!(inout->sequences))
    {
        ABORT (status, 
               STOCHASTICCONDITIONDATAH_ENULL, 
               STOCHASTICCONDITIONDATAH_MSGENULL);
    }
    
    
    /*
     *  
     * identify multiDimData sequences by ILWD comments, and check that 
     * the data has the correct type, dimensions, and length 
     *
     */
    
    for (i = 0; i < inout->numberSequences; ++i)
    {
      if ( !strcmp(inout->sequences[i].comment,
		   STOCHASTICCONDITIONDATAH_SPEC1COMMENT) )
      {
	spec1Index = i;

	/* check dimensions */
	if (inout->sequences[i].numberDimensions != 1)
	{
	  ABORT (status, 
		 STOCHASTICCONDITIONDATAH_EDIMS, 
		 STOCHASTICCONDITIONDATAH_MSGEDIMS);
	}
            
      }
      else if ( !strcmp(inout->sequences[i].comment,
			STOCHASTICCONDITIONDATAH_SPEC2COMMENT) )
      {
	spec2Index = i;
	
	/* check dimensions */
	if (inout->sequences[i].numberDimensions != 1)
	{
	  ABORT (status, 
		 STOCHASTICCONDITIONDATAH_EDIMS, 
		 STOCHASTICCONDITIONDATAH_MSGEDIMS);
	}
	
      }
      else if ( !strcmp(inout->sequences[i].comment,
			STOCHASTICCONDITIONDATAH_RESP1COMMENT) )
      {
	resp1Index = i;
	
	/* check dimensions */
	if (inout->sequences[i].numberDimensions != 1)
	{
	  ABORT (status, 
		 STOCHASTICCONDITIONDATAH_EDIMS, 
		 STOCHASTICCONDITIONDATAH_MSGEDIMS);
	}
	
      }
      else if ( !strcmp(inout->sequences[i].comment,
			  STOCHASTICCONDITIONDATAH_RESP2COMMENT) )
      {
	resp2Index = i;
	
	/* check dimensions */
	if (inout->sequences[i].numberDimensions != 1)
	{
	  ABORT (status, 
		 STOCHASTICCONDITIONDATAH_EDIMS, 
		 STOCHASTICCONDITIONDATAH_MSGEDIMS);
	}
	
      }
      else if ( !strcmp(inout->sequences[i].comment,
			STOCHASTICCONDITIONDATAH_DATA1COMMENT) )
      {
	data1Index = i;
	
	/* check dimensions */
	if (inout->sequences[i].numberDimensions != 1)
	{
	  ABORT (status, 
		 STOCHASTICCONDITIONDATAH_EDIMS, 
		 STOCHASTICCONDITIONDATAH_MSGEDIMS);
	}
	
	/* check length */
	if (inout->sequences[i].dimensions[0] 
	    != params->initParams->numSegments *
	    params->initParams->numPoints)
	{
	  ABORT (status, 
		 STOCHASTICCONDITIONDATAH_ESIZE, 
		 STOCHASTICCONDITIONDATAH_MSGESIZE);
	}

	/* Copy time interval for search output */
	params->timeInterval = inout->sequences[i].range.dTime;
      }
      else if ( !strcmp(inout->sequences[i].comment,
			  STOCHASTICCONDITIONDATAH_DATA2COMMENT) )
      {
	data2Index = i;
	
	/* check dimensions */
	if (inout->sequences[i].numberDimensions != 1)
	{
	  ABORT (status, 
		 STOCHASTICCONDITIONDATAH_EDIMS, 
		 STOCHASTICCONDITIONDATAH_MSGEDIMS);
	}
	
	/* check length */
	if (inout->sequences[i].dimensions[0] 
	    != params->initParams->numSegments *
	    params->initParams->numPoints)
	{
	  ABORT (status, 
		 STOCHASTICCONDITIONDATAH_ESIZE, 
		 STOCHASTICCONDITIONDATAH_MSGESIZE);
	}	
      }
      else
      {
	ABORT (status, 
	       STOCHASTICCONDITIONDATAH_ENAME, 
	       STOCHASTICCONDITIONDATAH_MSGENAME); 
      }
    }
    /* 
     *
     * allocate memory for non-trivial parameter fields 
     *
     */
   
    /* spec1 */
    spec1 = (REAL4FrequencySeries *) 
        LALMalloc (sizeof (REAL4FrequencySeries));
    if (!(spec1))
    {
        ABORT (status, 
               STOCHASTICCONDITIONDATAH_EALOC, 
               STOCHASTICCONDITIONDATAH_MSGEALOC);
    }
    
    spec1->data = NULL;
    TRY( LALCreateVector (status->statusPtr,
			  &(spec1->data),
			  params->initParams->numFrequencies),
	 status );
                    
    /* spec2 */    
    spec2 = (REAL4FrequencySeries *) 
        LALMalloc (sizeof (REAL4FrequencySeries));
    if (!(spec2))
    {
        ABORT (status, 
               STOCHASTICCONDITIONDATAH_EALOC, 
               STOCHASTICCONDITIONDATAH_MSGEALOC);
    }
    
    spec2->data = NULL;
    TRY( LALCreateVector (status->statusPtr,
			  &(spec2->data),
			  params->initParams->numFrequencies),
	 status );
    
    /* resp1 */    
    resp1 = (COMPLEX8FrequencySeries *) 
        LALMalloc (sizeof (COMPLEX8FrequencySeries));
    if (!(resp1))
    {
        ABORT (status, 
               STOCHASTICCONDITIONDATAH_EALOC, 
               STOCHASTICCONDITIONDATAH_MSGEALOC);
    }
   
    resp1->data = NULL;
    TRY( LALCCreateVector (status->statusPtr,
			   &(resp1->data),
			   params->initParams->numFrequencies),
	 status );
    
    /* resp2 */ 
    resp2 = (COMPLEX8FrequencySeries *) 
        LALMalloc (sizeof (COMPLEX8FrequencySeries));
    if (!(resp2))
    {
        ABORT (status, 
               STOCHASTICCONDITIONDATAH_EALOC, 
               STOCHASTICCONDITIONDATAH_MSGEALOC);
    }
    
    resp2->data = NULL;
    TRY( LALCCreateVector (status->statusPtr,
			   &(resp2->data),
			   params->initParams->numFrequencies),
	 status );

    /* calculate duration of a segment in seconds */

    params->segmentDuration = (REAL8) (params->initParams->numPoints)
      * inout->sequences[data1Index].range.dTime.timeStepSize;

    /*
     *
     * Fill in frequency sampling parameters from initparams
     *
     */

    freqParams.f0 = params->initParams->f0;
    freqParams.deltaF = params->initParams->deltaF;
    freqParams.length = params->initParams->numFrequencies;

    /* 
     *
     * translate spectrum InPut to REAL4FrequencySeries
     * and coarse grain to desired resolution
     *
     */
    
    /* spec1 */
    spec1TwoSidedData = NULL;
    TRY( LALSCreateVector(status->statusPtr,
			  &spec1TwoSidedData,
			  inout->sequences[spec1Index].dimensions[0]),
	 status );

    CastToReal4 ( status, spec1TwoSidedData->data,
		  &(inout->sequences[spec1Index]),
		  0, spec1TwoSidedData->length );

    /* check if this is a one- or two-sided spectrum */
    if ( inout->sequences[spec1Index].range.dFreq.startFreq >= 0) {
      spec1Raw.data = spec1TwoSidedData;
      spec1Raw.f0 = inout->sequences[spec1Index].range.dFreq.startFreq;
    }
    else 
    {
      offset 
	= (INT4) (- inout->sequences[spec1Index].range.dFreq.startFreq
		  / inout->sequences[spec1Index].range.dFreq.freqStepSize
		  + 0.5);
      spec1Raw.data = (REAL4Vector *)
	LALMalloc(sizeof(REAL4Vector));

      if (!(spec1Raw.data))
      {
	ABORT (status, 
	       STOCHASTICCONDITIONDATAH_EALOC, 
	       STOCHASTICCONDITIONDATAH_MSGEALOC);
      }

      spec1Raw.data->length = spec1TwoSidedData->length - offset;
      spec1Raw.data->data = spec1TwoSidedData->data + offset;
      spec1Raw.f0 = 0.0;
    }

    strncpy (spec1Raw.name, 
	     inout->sequences[spec1Index].name,
             LALNameLength * sizeof(CHAR));

    spec1Raw.deltaF = 
      inout->sequences[spec1Index].range.dFreq.freqStepSize;

    spec1Raw.epoch.gpsSeconds = 
      (inout->sequences[spec1Index].range.dFreq.gpsStartTimeSec);

    spec1Raw.epoch.gpsNanoSeconds = 
      (inout->sequences[spec1Index].range.dFreq.gpsStartTimeNan);

    /* really should get this from the ILWD */
    /*
    if ( strncmp(inout->sequences[spec1Index].units,"counts^2/Hz",
		 maxMultiDimUnits) && 
	 strncmp(inout->sequences[spec1Index].units,"count^2/Hz",
		 maxMultiDimUnits) &&
	 strncmp(inout->sequences[spec1Index].units,"counts^2/hz",
		 maxMultiDimUnits) && 
	 strncmp(inout->sequences[spec1Index].units,"count^2/hz",
		 maxMultiDimUnits) && 
	 strncmp(inout->sequences[spec1Index].units,"s count^2",
		 maxMultiDimUnits) &&
	 strncmp(inout->sequences[spec1Index].units,
		 inout->sequences[data1Index].units,
		 maxMultiDimUnits) ) {
      ABORT (status, 
	     STOCHASTICCONDITIONDATAH_EUNIT, 
	     STOCHASTICCONDITIONDATAH_MSGEUNIT);
    }
    */

    spec1Raw.sampleUnits = lalDimensionlessUnit;
    spec1Raw.sampleUnits.unitNumerator[LALUnitIndexADCCount] = 2;
    spec1Raw.sampleUnits.unitNumerator[LALUnitIndexSecond]   = 1;

    TRY( LALSCoarseGrainFrequencySeries(status->statusPtr,
					spec1,
					&spec1Raw,
					&freqParams),
	 status );

    if (spec1Raw.data != spec1TwoSidedData) {
      LALFree(spec1Raw.data);
    }

    TRY( LALSDestroyVector(status->statusPtr,
			   &spec1TwoSidedData),
	 status );    

    /* spec2 */
    spec2TwoSidedData = NULL;
    TRY( LALSCreateVector(status->statusPtr,
			  &spec2TwoSidedData,
			  inout->sequences[spec2Index].dimensions[0]),
	 status );

    CastToReal4 ( status, spec2TwoSidedData->data,
		  &(inout->sequences[spec2Index]),
		  0, spec2TwoSidedData->length );

    /* check if this is a one- or two-sided spectrum */
    if ( inout->sequences[spec2Index].range.dFreq.startFreq >= 0) {
      spec2Raw.data = spec2TwoSidedData;
      spec2Raw.f0 = inout->sequences[spec2Index].range.dFreq.startFreq;
    }
    else 
    {
      offset 
	= (INT4) (- inout->sequences[spec2Index].range.dFreq.startFreq
		  / inout->sequences[spec2Index].range.dFreq.freqStepSize
		  + 0.5);
      spec2Raw.data = (REAL4Vector *)
	LALMalloc(sizeof(REAL4Vector));

      if (!(spec2Raw.data))
      {
	ABORT (status, 
	       STOCHASTICCONDITIONDATAH_EALOC, 
	       STOCHASTICCONDITIONDATAH_MSGEALOC);
      }

      spec2Raw.data->length = spec2TwoSidedData->length - offset;
      spec2Raw.data->data = spec2TwoSidedData->data + offset;
      spec2Raw.f0 = 0.0;
    }

    strncpy (spec2Raw.name, 
	     inout->sequences[spec2Index].name,
             LALNameLength * sizeof(CHAR));

    spec2Raw.deltaF = 
        inout->sequences[spec2Index].range.dFreq.freqStepSize;

    spec2Raw.epoch.gpsSeconds = 
      (inout->sequences[spec2Index].range.dFreq.gpsStartTimeSec);

    spec2Raw.epoch.gpsNanoSeconds = 
      (inout->sequences[spec2Index].range.dFreq.gpsStartTimeNan);

    /* really should get this from the ILWD */
    /*
    if ( strncmp(inout->sequences[spec2Index].units,"counts^2/Hz",
		 maxMultiDimUnits) && 
	 strncmp(inout->sequences[spec2Index].units,"count^2/Hz",
		 maxMultiDimUnits) &&
	 strncmp(inout->sequences[spec2Index].units,"counts^2/hz",
		 maxMultiDimUnits) && 
	 strncmp(inout->sequences[spec2Index].units,"count^2/hz",
		 maxMultiDimUnits) && 
	 strncmp(inout->sequences[spec2Index].units,"s count^2",
		 maxMultiDimUnits) &&
	 strncmp(inout->sequences[spec2Index].units,
		 inout->sequences[data2Index].units,
		 maxMultiDimUnits) ) {
      ABORT (status, 
	     STOCHASTICCONDITIONDATAH_EUNIT, 
	     STOCHASTICCONDITIONDATAH_MSGEUNIT);
    }
    */

    spec2Raw.sampleUnits = lalDimensionlessUnit;
    spec2Raw.sampleUnits.unitNumerator[LALUnitIndexADCCount] = 2;
    spec2Raw.sampleUnits.unitNumerator[LALUnitIndexSecond]   = 1;

    TRY( LALSCoarseGrainFrequencySeries(status->statusPtr,
					spec2,
					&spec2Raw,
					&freqParams),
	 status );

    if (spec2Raw.data != spec2TwoSidedData) {
      LALFree(spec2Raw.data);
    }

    TRY( LALSDestroyVector(status->statusPtr,
			   &spec2TwoSidedData),
	 status );    
    
    /*
     *  
     *  translate response InPut to COMPLEX8FrequencySeries
     *  and coarse grain to desired resolution
     *
     */

    /* resp1 */
    resp1TwoSidedData = NULL;
    TRY( LALCCreateVector(status->statusPtr,
			  &resp1TwoSidedData,
			  inout->sequences[resp1Index].dimensions[0]),
	 status );

    CastToComplex8 ( status, resp1TwoSidedData->data,
		     &(inout->sequences[resp1Index]),
		     0, resp1TwoSidedData->length );

    /* need to discard redundant negative frequencies to make
     * LALCCoarseGrainFrequencySeries happy 
     */

    if ( inout->sequences[resp1Index].range.dFreq.startFreq >= 0) {
      resp1Raw.data = resp1TwoSidedData;
      resp1Raw.f0 = inout->sequences[resp1Index].range.dFreq.startFreq;
    }
    else 
    {
      offset 
	= (INT4) (- inout->sequences[resp1Index].range.dFreq.startFreq
		  / inout->sequences[resp1Index].range.dFreq.freqStepSize
		  + 0.5);
      resp1Raw.data = (COMPLEX8Vector *)
	LALMalloc(sizeof(COMPLEX8Vector));

      if (!(resp1Raw.data))
      {
	ABORT (status, 
	       STOCHASTICCONDITIONDATAH_EALOC, 
	       STOCHASTICCONDITIONDATAH_MSGEALOC);
      }

      resp1Raw.data->length = resp1TwoSidedData->length - offset;
      resp1Raw.data->data = resp1TwoSidedData->data + offset;
      resp1Raw.f0 = 0.0;
    }

    strncpy (resp1Raw.name, 
	     inout->sequences[resp1Index].name,
             LALNameLength * sizeof(CHAR));

    resp1Raw.deltaF = 
        inout->sequences[resp1Index].range.dFreq.freqStepSize;

    resp1Raw.epoch.gpsSeconds = 
      (inout->sequences[resp1Index].range.dFreq.gpsStartTimeSec);

    resp1Raw.epoch.gpsNanoSeconds = 
      (inout->sequences[resp1Index].range.dFreq.gpsStartTimeNan);

    /* really should get this from the ILWD */
    /*
    if ( strncmp(inout->sequences[resp1Index].units,"count/attostrain",
		 maxMultiDimUnits) && 
	 strncmp(inout->sequences[resp1Index].units,"10^18 ADC count/strain",
		 maxMultiDimUnits) && 
	 strncmp(inout->sequences[resp1Index].units,"10^18 strain^-1 count",
		 maxMultiDimUnits) ) {
      ABORT (status, 
	     STOCHASTICCONDITIONDATAH_EUNIT, 
	     STOCHASTICCONDITIONDATAH_MSGEUNIT);
    }
    */
    resp1Raw.sampleUnits = lalADCCountUnit;
    resp1Raw.sampleUnits.unitNumerator[LALUnitIndexStrain]   = -1;
    resp1Raw.sampleUnits.powerOfTen = 18;

    TRY( LALCCoarseGrainFrequencySeries(status->statusPtr,
					resp1,
					&resp1Raw,
					&freqParams),
	 status );

    if (resp1Raw.data != resp1TwoSidedData) {
      LALFree(resp1Raw.data);
    }

    TRY( LALCDestroyVector(status->statusPtr,
			   &resp1TwoSidedData),
	 status );    

    /* resp2 */
    resp2TwoSidedData = NULL;
    TRY( LALCCreateVector(status->statusPtr,
			  &resp2TwoSidedData,
			  inout->sequences[resp2Index].dimensions[0]),
	 status );

    CastToComplex8 ( status, resp2TwoSidedData->data,
		     &(inout->sequences[resp2Index]),
		     0, resp2TwoSidedData->length );

    /* need to discard redundant negative frequencies to make
     * LALCCoarseGrainFrequencySeries happy 
     */

    if ( inout->sequences[resp2Index].range.dFreq.startFreq >= 0) {
      resp2Raw.data = resp2TwoSidedData;
      resp2Raw.f0 = inout->sequences[resp2Index].range.dFreq.startFreq;
    }
    else 
    {
      offset 
	= (INT4) (- inout->sequences[resp2Index].range.dFreq.startFreq
		  / inout->sequences[resp2Index].range.dFreq.freqStepSize
		  + 0.5);
      resp2Raw.data = (COMPLEX8Vector *)
	LALMalloc(sizeof(COMPLEX8Vector));

      if (!(resp2Raw.data))
      {
	ABORT (status, 
	       STOCHASTICCONDITIONDATAH_EALOC, 
	       STOCHASTICCONDITIONDATAH_MSGEALOC);
      }

      resp2Raw.data->length = resp2TwoSidedData->length - offset;
      resp2Raw.data->data = resp2TwoSidedData->data + offset;
      resp2Raw.f0 = 0.0;
    }

    strncpy (resp2Raw.name, 
	     inout->sequences[resp2Index].name,
             LALNameLength * sizeof(CHAR));

    resp2Raw.deltaF = 
        inout->sequences[resp2Index].range.dFreq.freqStepSize;

    resp2Raw.epoch.gpsSeconds = 
      (inout->sequences[resp2Index].range.dFreq.gpsStartTimeSec);

    resp2Raw.epoch.gpsNanoSeconds = 
      (inout->sequences[resp2Index].range.dFreq.gpsStartTimeNan);

    /* really should get this from the ILWD */
    /*
    if ( strncmp(inout->sequences[resp2Index].units,"count/attostrain",
		 maxMultiDimUnits) && 
	 strncmp(inout->sequences[resp2Index].units,"10^18 ADC count/strain",
		 maxMultiDimUnits) && 
	 strncmp(inout->sequences[resp2Index].units,"10^18 strain^-1 count",
		 maxMultiDimUnits) ) {
      ABORT (status, 
	     STOCHASTICCONDITIONDATAH_EUNIT, 
	     STOCHASTICCONDITIONDATAH_MSGEUNIT);
    }
    */
    resp2Raw.sampleUnits = lalADCCountUnit;
    resp2Raw.sampleUnits.unitNumerator[LALUnitIndexStrain]   = -1;
    resp2Raw.sampleUnits.powerOfTen = 18;

    TRY( LALCCoarseGrainFrequencySeries(status->statusPtr,
					resp2,
					&resp2Raw,
					&freqParams),
	 status );

    if (resp2Raw.data != resp2TwoSidedData) {
      LALFree(resp2Raw.data);
    }

    TRY( LALCDestroyVector(status->statusPtr,
			   &resp2TwoSidedData),
	 status );    


    /* We need the window even if we're not cross-correlating, for
       normalization putposes */

    /* First window */
    windowParams.type = params->initParams->windowType1;
    if ( params->initParams->windowTop1 == 0 
	 || windowParams.type == Rectangular )
    {
      windowParams.length = params->initParams->numPoints;
      TRY( LALWindow(status->statusPtr, params->window1, &windowParams),
	   status );
    }
    else
    {
      windowParams.length = params->initParams->numPoints 
	- params->initParams->windowTop1;
      if ( windowParams.length )
      {
	window = NULL;
	TRY( LALSCreateVector(status->statusPtr,
			      &window,
			      windowParams.length),
	     status );
	TRY( LALWindow(status->statusPtr, window, &windowParams),
	     status );
	/* The number of points in the first half of the window, before
	   the flat top */
	halfWin = (windowParams.length + 1) / 2;
	sPtr = 
	  memcpy ( params->window1->data, window->data,
		   halfWin * sizeof(REAL4) );
	if (sPtr != params->window1->data)
	{
	  ABORT (status, 
		 STOCHASTICCONDITIONDATAH_EALOC, 
		 STOCHASTICCONDITIONDATAH_MSGEALOC);
	}
      }
      else
      {
	halfWin = 0;
      }

      for (sPtr = params->window1->data+halfWin,
	     sStopPtr = sPtr + params->initParams->windowTop1;
	   sPtr < sStopPtr; ++sPtr)
      {
	*sPtr = 1.0;
      }

      if ( windowParams.length )
      {
	sPtr = 
	  memcpy ( sStopPtr, window->data + halfWin, 
		   (windowParams.length-halfWin) * sizeof(REAL4) );
	if (sPtr != sStopPtr)
	{
	  ABORT (status, 
		 STOCHASTICCONDITIONDATAH_EALOC, 
		 STOCHASTICCONDITIONDATAH_MSGEALOC);
	}
	TRY( LALSDestroyVector(status->statusPtr,
			       &window),
	     status );
      }
    }

    /* Second window */
    windowParams.type = params->initParams->windowType2;
    if ( params->initParams->windowTop2 == 0 
	 || windowParams.type == Rectangular )
    {
      windowParams.length = params->initParams->numPoints;
      TRY( LALWindow(status->statusPtr, params->window2, &windowParams),
	   status );
    }
    else
    {
      windowParams.length = params->initParams->numPoints 
	- params->initParams->windowTop2;

      if ( windowParams.length )
      {
	window = NULL;
	TRY( LALSCreateVector(status->statusPtr,
			      &window,
			      windowParams.length),
	     status );
	TRY( LALWindow(status->statusPtr, window, &windowParams),
	     status );
	/* The number of points in the first half of the window, before
	   the flat top */
	halfWin = (windowParams.length + 1) / 2;
	sPtr = 
	  memcpy ( params->window2->data, window->data,
		   halfWin * sizeof(REAL4) );
	if (sPtr != params->window2->data)
	{
	  ABORT (status, 
		 STOCHASTICCONDITIONDATAH_EALOC, 
		 STOCHASTICCONDITIONDATAH_MSGEALOC);
	}
      }
      else
      {
	halfWin = 0;
      }

      for (sPtr = params->window2->data+halfWin,
	     sStopPtr = sPtr + params->initParams->windowTop2;
	   sPtr < sStopPtr; ++sPtr)
      {
	*sPtr = 1.0;
      }

      if ( windowParams.length )
      {
	sPtr = 
	  memcpy ( sStopPtr, window->data + halfWin, 
		   (windowParams.length-halfWin) * sizeof(REAL4) );
	if (sPtr != sStopPtr)
	{
	  ABORT (status, 
		 STOCHASTICCONDITIONDATAH_EALOC, 
		 STOCHASTICCONDITIONDATAH_MSGEALOC);
	}
	TRY( LALSDestroyVector(status->statusPtr,
			       &window),
	     status );
      }
    }

    /* The following conditional means we only do this if we're
       calculating the cross-correlaiton */

    if ( params->optimalFilter )
    {

      /* 
       * translate InPut data to DataSegment 
       */ 
      
      if (!(params->initParams->baseFreq))
      {   // Real data
	
	/* This check is mostly to make sure non-heterodyned data are real */
	if (inout->sequences[data1Index].type != real_4) 
	{
	  ABORT (status, 
		 STOCHASTICCONDITIONDATAH_ETYPE, 
		 STOCHASTICCONDITIONDATAH_MSGETYPE);
	}
	if (inout->sequences[data2Index].type != real_4) 
	{
	  ABORT (status, 
		 STOCHASTICCONDITIONDATAH_ETYPE, 
		 STOCHASTICCONDITIONDATAH_MSGETYPE);
	}
	
	sDataSegVec = params->stochasticSegVec.realVec;
	
	for (i = 0; i < sDataSegVec->length; ++i)
	{
	  /* point to current segment */
	  sDummySegment = sDataSegVec->data + i;
	  
	  /* 
	   * copy the data 
	   */
	  
	  /* data1 */
	  CastToReal4 ( status, sDummySegment->data1->data->data,
			&(inout->sequences[data1Index]),
			i * params->initParams->numPoints,
			params->initParams->numPoints );
	  
	  sDummySegment->data1->data->length = params->initParams->numPoints;
	  
	  strncpy (sDummySegment->data1->name,
		   inout->sequences[data1Index].name,
		   LALNameLength * sizeof(CHAR));
	  
	  sDummySegment->data1->deltaT = 
            inout->sequences[data1Index].range.dTime.timeStepSize;
	  
	  epochNanoSeconds = 1000000000L
	    * (INT8) inout->sequences[data1Index].range.dTime.startSec;
	  epochNanoSeconds 
	    += (INT8) inout->sequences[data1Index].range.dTime.startNan;
	  epochNanoSeconds
	    += (INT8) ( 1e9 * (REAL8) i 
			* params->segmentDuration );
	  
	  sDummySegment->data1->epoch.gpsSeconds
	    = (INT4) (epochNanoSeconds / 1000000000L);
	  sDummySegment->data1->epoch.gpsNanoSeconds
	    = (INT4) (epochNanoSeconds % 1000000000L);
	  
	  sDummySegment->data1->f0 = 0.0;
	  
	  /* really should get this from the ILWD */
	  
	  sDummySegment->data1->sampleUnits = lalADCCountUnit;
	  
	  /* data2 */
	  CastToReal4 ( status, sDummySegment->data2->data->data,
			&(inout->sequences[data2Index]),
			i * params->initParams->numPoints,
			params->initParams->numPoints );
	  
	  sDummySegment->data2->data->length = params->initParams->numPoints;
	  
	  strncpy (sDummySegment->data2->name,
		   inout->sequences[data2Index].name,
		   LALNameLength * sizeof(CHAR));
	  
	  sDummySegment->data2->deltaT = 
            inout->sequences[data2Index].range.dTime.timeStepSize;
	  
	  epochNanoSeconds = 1000000000L
	    * (INT8) inout->sequences[data2Index].range.dTime.startSec;
	  epochNanoSeconds 
	    += (INT8) inout->sequences[data2Index].range.dTime.startNan;
	  epochNanoSeconds
	    += (INT8) ( 1e9 * (REAL8) i 
			* params->segmentDuration );
	  
	  sDummySegment->data2->epoch.gpsSeconds
	    = (INT4) (epochNanoSeconds / 1000000000L);
	  sDummySegment->data2->epoch.gpsNanoSeconds
	    = (INT4) (epochNanoSeconds % 1000000000L);
	  
	  sDummySegment->data2->f0 = 0.0;
	  
	  /* really should get this from the ILWD */
	  
	  sDummySegment->data2->sampleUnits = lalADCCountUnit;
	}
      } /* end real data option */
      else 
      {   // Complex data
	/* This check is mostly to make sure heterodyned data are complex */
	if (inout->sequences[data1Index].type != complex_8) 
	{
	  ABORT (status, 
		 STOCHASTICCONDITIONDATAH_ETYPE, 
		 STOCHASTICCONDITIONDATAH_MSGETYPE);
	}
	if (inout->sequences[data2Index].type != complex_8) 
	{
	  ABORT (status, 
		 STOCHASTICCONDITIONDATAH_ETYPE, 
		 STOCHASTICCONDITIONDATAH_MSGETYPE);
	}
	
	cDataSegVec = params->stochasticSegVec.complexVec;
	
	for (i = 0; i < cDataSegVec->length; ++i)
	{
	  /* point to current segment */
	  cDummySegment = cDataSegVec->data + i;
	  
	  /* 
	   * copy the data 
	   */
	  
	  /* data1 */
	  CastToComplex8 ( status, cDummySegment->data1->data->data,
			   &(inout->sequences[data1Index]),
			   i * params->initParams->numPoints,
			   params->initParams->numPoints );
	  
	  cDummySegment->data1->data->length = params->initParams->numPoints;
	  
	  strncpy (cDummySegment->data1->name,
		   inout->sequences[data1Index].name,
		   LALNameLength * sizeof(CHAR));
	  
	  cDummySegment->data1->deltaT = 
            inout->sequences[data1Index].range.dTime.timeStepSize;
	  
	  epochNanoSeconds = 1000000000L
	    * (INT8) inout->sequences[data1Index].range.dTime.startSec;
	  epochNanoSeconds 
	    += (INT8) inout->sequences[data1Index].range.dTime.startNan;
	  epochNanoSeconds
	    += (INT8) ( 1e9 * (REAL8) i 
			* params->segmentDuration );
	  
	  cDummySegment->data1->epoch.gpsSeconds
	    = (INT4) (epochNanoSeconds / 1000000000L);
	  cDummySegment->data1->epoch.gpsNanoSeconds
	    = (INT4) (epochNanoSeconds % 1000000000L);
	  
	  cDummySegment->data1->f0 = params->initParams->baseFreq;
	  
	  /* really should get this from the ILWD */
	  
	  cDummySegment->data1->sampleUnits = lalADCCountUnit;
	  
	  /* data2 */
	  CastToComplex8 ( status, cDummySegment->data2->data->data,
			   &(inout->sequences[data2Index]),
			   i * params->initParams->numPoints,
			   params->initParams->numPoints );
	  
	  cDummySegment->data2->data->length = params->initParams->numPoints;
	  
	  strncpy (cDummySegment->data2->name,
		   inout->sequences[data2Index].name,
		   LALNameLength * sizeof(CHAR));
	  
	  cDummySegment->data2->deltaT = 
            inout->sequences[data2Index].range.dTime.timeStepSize;
	  
	  epochNanoSeconds = 1000000000L
	    * (INT8) inout->sequences[data2Index].range.dTime.startSec;
	  epochNanoSeconds 
	    += (INT8) inout->sequences[data2Index].range.dTime.startNan;
	  epochNanoSeconds
	    += (INT8) ( 1e9 * (REAL8) i 
			* params->segmentDuration );
	  
	  cDummySegment->data2->epoch.gpsSeconds
	    = (INT4) (epochNanoSeconds / 1000000000L);
	  cDummySegment->data2->epoch.gpsNanoSeconds
	    = (INT4) (epochNanoSeconds % 1000000000L);
	  
	  cDummySegment->data2->f0 = params->initParams->baseFreq;
	  
	  /* really should get this from the ILWD */
	  
	  cDummySegment->data2->sampleUnits = lalADCCountUnit;
	}
      } /* end complex data option */
    }
    
    /* 
     *
     * calculate overlap reduction function 
     *
     */

    /* allocate memory for frequency series */
    overlap.data = NULL;
    TRY( LALCreateVector (status->statusPtr,
			  &(overlap.data),
			  params->initParams->numFrequencies),
	 status );

    /* fill in OverlapReductionFunctionParameters structure */
    overlapParams.length = params->initParams->numFrequencies;
    overlapParams.f0     = params->initParams->f0;
    overlapParams.deltaF = params->initParams->deltaF;

    /* Determine type or ID of first detector */
    switch (params->initParams->siteID1)
    {
    case 'i': case 'b': // Get geometry from frame
      geometry = inout->sequences[data1Index].range.dTime.geometry;
      strncpy (frDetector.name,geometry->name, sizeof(frDetector.name));
      /* LDAS and LAL now measure lat/lon in radians */
      frDetector.vertexLongitudeRadians = geometry->longitude;
      frDetector.vertexLatitudeRadians = geometry->latitude;
      frDetector.vertexElevation = geometry->elevation;
      /* LDAS and LAL now measure azimuth clockwise from North; */
      frDetector.xArmAzimuthRadians = geometry->armXazimuth;
      frDetector.xArmAltitudeRadians = geometry->armXaltitude;
      frDetector.yArmAzimuthRadians = geometry->armYazimuth;
      frDetector.yArmAltitudeRadians = geometry->armYaltitude;
      switch (params->initParams->siteID1)
      {
      case 'i':
	TRY( LALCreateDetector( status->statusPtr, &(detectorPair.detectorOne),
				&frDetector, LALDETECTORTYPE_IFODIFF ),
	     status );
	break;
      case 'b':
	TRY( LALCreateDetector( status->statusPtr, &(detectorPair.detectorOne),
				&frDetector, LALDETECTORTYPE_CYLBAR ),
	     status );
	break;
      }
      break;
    case 'H': // LIGO Hanford
      detectorPair.detectorOne = lalCachedDetectors[LALDetectorIndexLHODIFF];
      if (params->initParams->azimuth1)
      {
	ABORT ( status,
		STOCHASTICCONDITIONDATAH_ESTID, 
		STOCHASTICCONDITIONDATAH_MSGESTID );
      }
      break;
    case 'L': // LIGO Livingston
      detectorPair.detectorOne = lalCachedDetectors[LALDetectorIndexLLODIFF];
      if (params->initParams->azimuth1)
      {
	ABORT ( status,
		STOCHASTICCONDITIONDATAH_ESTID, 
		STOCHASTICCONDITIONDATAH_MSGESTID );
      }
      break;
    case 'G': // GEO-600
      detectorPair.detectorOne
	= lalCachedDetectors[LALDetectorIndexGEO600DIFF];
      if (params->initParams->azimuth1)
      {
	ABORT ( status,
		STOCHASTICCONDITIONDATAH_ESTID, 
		STOCHASTICCONDITIONDATAH_MSGESTID );
      }
      break;
    case 'A': // ALLEGRO
      strncpy (frDetector.name, "ALLEGRO", sizeof(frDetector.name));
      frDetector.vertexLongitudeRadians
	= STOCHASTICCONDITIONDATAC_ALLEGROLON;
      frDetector.vertexLatitudeRadians
	= STOCHASTICCONDITIONDATAC_ALLEGROLAT;
      frDetector.xArmAltitudeRadians = 0;
      if (params->initParams->azimuth1)
      {
	frDetector.xArmAzimuthRadians
	  = *(params->initParams->azimuth1) * LAL_PI_180;
      }
      else 
      {
	frDetector.xArmAzimuthRadians = STOCHASTICCONDITIONDATAC_ALLEGROAZ;
      }
      TRY( LALCreateDetector( status->statusPtr, &(detectorPair.detectorOne),
			      &frDetector, LALDETECTORTYPE_CYLBAR ),
	   status );
      break;
    case '0': case '1': case '2': case '3': case '4':
    case '5': case '6': case '7': case '8': case '9':
      if (params->initParams->azimuth1)
      {
	ABORT ( status,
		STOCHASTICCONDITIONDATAH_ESTID, 
		STOCHASTICCONDITIONDATAH_MSGESTID );
      }
      detInd = (UINT4) ctoi(params->initParams->siteID1);
      // printf("%c %d\n",params->initParams->siteID1, detInd);
      if ( detInd >= 0 && detInd < LALNumCachedDetectors )
      {
	detectorPair.detectorOne = lalCachedDetectors[detInd];
	break;
      }
      // Fall through if out of range
    default:
      ABORT ( status,
	      STOCHASTICCONDITIONDATAH_ESTID, 
	      STOCHASTICCONDITIONDATAH_MSGESTID );
    }

    /* Determine type or ID of second detector */
    switch (params->initParams->siteID2)
    {
    case 'i': case 'b': // Get geometry from frame
      geometry = inout->sequences[data2Index].range.dTime.geometry;
      strncpy (frDetector.name,geometry->name, sizeof(frDetector.name));
      /* LDAS and LAL now measure lat/lon in radians */
      frDetector.vertexLongitudeRadians = geometry->longitude;
      frDetector.vertexLatitudeRadians = geometry->latitude;
      frDetector.vertexElevation = geometry->elevation;
      /* LDAS and LAL now measure azimuth clockwise from North; */
      frDetector.xArmAzimuthRadians = geometry->armXazimuth;
      frDetector.xArmAltitudeRadians = geometry->armXaltitude;
      frDetector.yArmAzimuthRadians = geometry->armYazimuth;
      frDetector.yArmAltitudeRadians = geometry->armYaltitude;
      switch (params->initParams->siteID2)
      {
      case 'i':
	TRY( LALCreateDetector( status->statusPtr, &(detectorPair.detectorTwo),
				&frDetector, LALDETECTORTYPE_IFODIFF ),
	     status );
	break;
      case 'b':
	TRY( LALCreateDetector( status->statusPtr, &(detectorPair.detectorTwo),
				&frDetector, LALDETECTORTYPE_CYLBAR ),
	     status );
	break;
      }
      break;
    case 'H': // LIGO Hanford
      detectorPair.detectorTwo = lalCachedDetectors[LALDetectorIndexLHODIFF];
      if (params->initParams->azimuth2)
      {
	ABORT ( status,
		STOCHASTICCONDITIONDATAH_ESTID, 
		STOCHASTICCONDITIONDATAH_MSGESTID );
      }
      break;
    case 'L': // LIGO Livingston
      detectorPair.detectorTwo = lalCachedDetectors[LALDetectorIndexLLODIFF];
      if (params->initParams->azimuth2)
      {
	ABORT ( status,
		STOCHASTICCONDITIONDATAH_ESTID, 
		STOCHASTICCONDITIONDATAH_MSGESTID );
      }
      break;
    case 'G': // GEO-600
      detectorPair.detectorTwo 
	= lalCachedDetectors[LALDetectorIndexGEO600DIFF];
      if (params->initParams->azimuth2)
      {
	ABORT ( status,
		STOCHASTICCONDITIONDATAH_ESTID, 
		STOCHASTICCONDITIONDATAH_MSGESTID );
      }
      break;
    case 'A': // ALLEGRO
      strncpy (frDetector.name, "ALLEGRO", sizeof(frDetector.name));
      frDetector.vertexLongitudeRadians
	= STOCHASTICCONDITIONDATAC_ALLEGROLON;
      frDetector.vertexLatitudeRadians
	= STOCHASTICCONDITIONDATAC_ALLEGROLAT;
      frDetector.xArmAltitudeRadians = 0;
      if (params->initParams->azimuth2)
      {
	frDetector.xArmAzimuthRadians
	  = *(params->initParams->azimuth2) * LAL_PI_180;
      }
      else 
      {
	frDetector.xArmAzimuthRadians = STOCHASTICCONDITIONDATAC_ALLEGROAZ;
      }
      TRY( LALCreateDetector( status->statusPtr, &(detectorPair.detectorTwo),
			      &frDetector, LALDETECTORTYPE_CYLBAR ),
	   status );
      break;
    case '0': case '1': case '2': case '3': case '4':
    case '5': case '6': case '7': case '8': case '9':
      if (params->initParams->azimuth2)
      {
	ABORT ( status,
		STOCHASTICCONDITIONDATAH_ESTID, 
		STOCHASTICCONDITIONDATAH_MSGESTID );
      }
      detInd = (UINT4) ctoi(params->initParams->siteID2);
      // printf("%c %d\n",params->initParams->siteID2, detInd);
      if ( detInd >= 0 && detInd < LALNumCachedDetectors )
      {
	detectorPair.detectorTwo = lalCachedDetectors[detInd];
	break;
      }
      // Fall through if out of range
    default:
      ABORT ( status,
	      STOCHASTICCONDITIONDATAH_ESTID, 
	      STOCHASTICCONDITIONDATAH_MSGESTID );
    }

    /* calculate overlap reduction function values */
#ifdef STOCHASTIC_SO_DEBUG
    PrintLALDetector(&detectorPair.detectorOne);
    PrintLALDetector(&detectorPair.detectorTwo);
#endif
    TRY( LALOverlapReductionFunction (status->statusPtr,
				      &overlap,
				      &detectorPair,
				      &overlapParams),
	 status );

    /*
     *
     * set omega_GW = const 
     *
     */
    
    /* allocate memory for frequency series */
    omegaGW.data = NULL;
    TRY( LALCreateVector (status->statusPtr,
			  &(omegaGW.data),
			  params->initParams->numFrequencies),
	 status );

    /* fill in StochasticOmegaGWParameters structure */
    omegaGWParams.alpha    = 0;
    omegaGWParams.fRef     = params->initParams->f0
      + ( ( params->initParams->numFrequencies - 1 ) / 2
	  * params->initParams->deltaF );
    omegaGWParams.omegaRef = 1;
    omegaGWParams.length   = params->initParams->numFrequencies;
    omegaGWParams.f0       = params->initParams->f0;
    omegaGWParams.deltaF   = params->initParams->deltaF;

    /* calculate omegaGW */
    TRY( LALStochasticOmegaGW (status->statusPtr,
			       &omegaGW,
			       &omegaGWParams),
	 status );
    
    /* 
     *
     * calculate inverse noise psds 
     *
     */

    /* spec1 */
    /* allocate memory for StochasticInverseNoiseOutput fields */
    calibratedInvNoise1.data = NULL;
    TRY( LALCreateVector (status->statusPtr,
			  &(calibratedInvNoise1.data),
			  params->initParams->numFrequencies),
	 status );
    
    halfCalibratedInvNoise1.data = NULL;
    TRY( LALCCreateVector (status->statusPtr,
			   &(halfCalibratedInvNoise1.data),
			   params->initParams->numFrequencies),
	 status );
    
    /* fill in StochasticInverseNoiseInput fields */
    invNoiseInput1.unCalibratedNoisePSD = spec1;
    invNoiseInput1.responseFunction  = resp1;
    
    /* fill in StochasticInverseNoiseOutput fields */
    invNoiseOutput1.calibratedInverseNoisePSD   = &calibratedInvNoise1;
    invNoiseOutput1.halfCalibratedInverseNoisePSD = &halfCalibratedInvNoise1;

    /* calculate inverse noise outputs */
    TRY( LALStochasticInverseNoise (status->statusPtr,
				    &invNoiseOutput1,
				    &invNoiseInput1),
	 status );
     
    /* spec2 */
    /* allocate memory for StochasticInverseNoiseOutput fields */
    calibratedInvNoise2.data = NULL;
    TRY( LALCreateVector (status->statusPtr,
			  &(calibratedInvNoise2.data),
			  params->initParams->numFrequencies),
	 status );
    
    halfCalibratedInvNoise2.data = NULL;
    TRY( LALCCreateVector (status->statusPtr,
			   &(halfCalibratedInvNoise2.data),
			   params->initParams->numFrequencies),
	 status );
    
    /* fill in StochasticInverseNoiseInput fields */
    invNoiseInput2.unCalibratedNoisePSD = spec2;
    invNoiseInput2.responseFunction  = resp2;
    
    /* fill in StochasticInverseNoiseOutput fields */
    invNoiseOutput2.calibratedInverseNoisePSD   = &calibratedInvNoise2;
    invNoiseOutput2.halfCalibratedInverseNoisePSD = &halfCalibratedInvNoise2;

    /* calculate inverse noise outputs */
    TRY( LALStochasticInverseNoise (status->statusPtr,
				    &invNoiseOutput2,
				    &invNoiseInput2),
	 status );

    /*
     *
     * calculate optimal filter normalization
     *
     */
    
    /* fill in StochasticOptimalFilterNormalizationInput fields */
    optNormInput.overlapReductionFunction = &overlap;
    optNormInput.omegaGW = &omegaGW;
    optNormInput.inverseNoisePSD1 = &calibratedInvNoise1;
    optNormInput.inverseNoisePSD2 = &calibratedInvNoise2;

    /* fill in StochasticOptimalFilterNormalizationParameters fields */
    optNormParams.fRef = omegaGWParams.fRef;

    /* check base frequency for heterodyned data */
    optNormParams.heterodyned = (BOOLEAN) params->initParams->baseFreq;

    /* Specify window functions for normalization */
    optNormParams.window1 = params->window1;
    optNormParams.window2 = params->window2;

    /* fill in StochasticOptimalFilterNormalizationParameters fields */
    if ( params->optimalFilter ) {
      optNormOutput.normalization = &lambda;
    }
    else
    {
      optNormOutput.normalization = NULL;
    }

    /* params->sigma is null if sigma has not been requested */
    optNormOutput.variance      = params->sigma;

    /* calculate optimal filter normalization */

    TRY( LALStochasticOptimalFilterNormalization (status->statusPtr,
						  &optNormOutput,
						  &optNormInput,
						  &optNormParams),
	 status );
    
    /* This is sigma^2/T, so we need to convert it to sigma */ 
    /* Corresponding to a single segment */

    if (params->sigma)
    {
      params->sigma->value = sqrt(params->sigma->value
				  * params->segmentDuration );
      unitPair.unitOne = &(params->sigma->units);
      unitPair.unitTwo = &lalSecondUnit;
      TRY( LALUnitMultiply(status->statusPtr,
			   &tmpUnit,
			   &unitPair),
	   status );
      power.numerator = 1;
      power.denominatorMinusOne = 1;
      TRY( LALUnitRaise(status->statusPtr,
			&(params->sigma->units),
			&tmpUnit,
			&power),
	   status );
    }

    if ( params->optimalFilter ) {
      /*
       *
       * calculate optimal filter 
       *
       */
      
      /* fill in StochasticOptimalFilterInput fields */
      optimalInput.overlapReductionFunction = &overlap;
      optimalInput.omegaGW = &omegaGW;
      optimalInput.halfCalibratedInverseNoisePSD1 = &halfCalibratedInvNoise1;
      optimalInput.halfCalibratedInverseNoisePSD2 = &halfCalibratedInvNoise2;
      
      /* calculate optimal filter values */
      
      TRY( LALStochasticOptimalFilter (status->statusPtr,
				       params->optimalFilter,
				       &optimalInput,
				       &lambda),
	   status );
    }
    else
    {
      params->optimalFilter = NULL;
    }

    /*
     *
     * clean up
     *
     */
    
    TRY( LALCDestroyVector (status->statusPtr,
			    &(halfCalibratedInvNoise2.data)),
	 status );
    
    TRY( LALDestroyVector (status->statusPtr, &(calibratedInvNoise2.data)),
	 status );
    
    TRY( LALCDestroyVector (status->statusPtr,
			    &(halfCalibratedInvNoise1.data)),
	 status );
    
    TRY( LALDestroyVector (status->statusPtr, &(calibratedInvNoise1.data)),
	 status );
    
    TRY( LALDestroyVector (status->statusPtr, &(omegaGW.data)),
	 status );
    
    TRY( LALDestroyVector (status->statusPtr, &(overlap.data)),
	 status );

    TRY( LALDestroyVector (status->statusPtr, &(spec1->data)),
	 status );

    LALFree(spec1);

    TRY( LALDestroyVector (status->statusPtr, &(spec2->data)),
	 status );

    LALFree(spec2);

    TRY( LALCDestroyVector (status->statusPtr, &(resp1->data)),
	 status );

    LALFree(resp1);

    TRY( LALCDestroyVector (status->statusPtr, &(resp2->data)),
	 status );

    LALFree(resp2);
    
    /* normal exit */
    DETATCHSTATUSPTR (status);
    RETURN (status);
}

static void CastToReal4 ( LALStatus *status, REAL4 *output,
			  const multiDimData *input,
			  const UINT4 offset, const UINT4 length )
{
  REAL4     *sPtr, *sStopPtr;
  REAL8     *dPtr;
  INT2      *i2Ptr;
  UINT2     *u2Ptr;
  INT4      *i4Ptr;
  UINT4     *u4Ptr;
  INT8      *i8Ptr;
  UINT8     *u8Ptr;
  COMPLEX8  *cPtr;
  COMPLEX16 *zPtr;

  /* printf ("%d %d\n", input->type, real_4); */
  switch (input->type)
  {
    case real_4:
      sPtr = 
	memcpy ( output, input->data.real4 + offset,
		 length * sizeof(REAL4) );
      if (sPtr != output)
      {
        ABORT (status, 
               STOCHASTICCONDITIONDATAH_EALOC, 
               STOCHASTICCONDITIONDATAH_MSGEALOC);
      }
      break;
    case real_8:
      sStopPtr = output + length;
      for ( sPtr = output, dPtr = input->data.real8 + offset;
	    sPtr < sStopPtr; ++sPtr, ++dPtr )
      {
	*sPtr = (REAL4) *dPtr;
      }
    break;
    case int_2s:
      sStopPtr = output + length;
      for ( sPtr = output, i2Ptr = input->data.int2s + offset;
	    sPtr < sStopPtr; ++sPtr, ++i2Ptr )
      {
	*sPtr = (REAL4) *i2Ptr;
      }
    break;
    case int_2u:
      sStopPtr = output + length;
      for ( sPtr = output, u2Ptr = input->data.int2u + offset;
	    sPtr < sStopPtr; ++sPtr, ++u2Ptr )
      {
	*sPtr = (REAL4) *u2Ptr;
      }
    break;
    case int_4u:
      sStopPtr = output + length;
      for ( sPtr = output, u4Ptr = input->data.int4u + offset;
	    sPtr < sStopPtr; ++sPtr, ++u4Ptr )
      {
	*sPtr = (REAL4) *u4Ptr;
      }
    break;
    case int_4s:
      sStopPtr = output + length;
      for ( sPtr = output, i4Ptr = input->data.int4s + offset;
	    sPtr < sStopPtr; ++sPtr, ++i4Ptr )
      {
	*sPtr = (REAL4) *i4Ptr;
      }
    break;
    case int_8u:
      sStopPtr = output + length;
      for ( sPtr = output, u8Ptr = input->data.int8u + offset;
	    sPtr < sStopPtr; ++sPtr, ++u8Ptr )
      {
	*sPtr = (REAL4) *u8Ptr;
      }
    break;
    case int_8s:
      sStopPtr = output + length;
      for ( sPtr = output, i8Ptr = input->data.int8s + offset;
	    sPtr < sStopPtr; ++sPtr, ++i8Ptr )
      {
	*sPtr = (REAL4) *i8Ptr;
      }
    break;
    case complex_8:
      sStopPtr = output + length;
      for ( sPtr = output, cPtr = input->data.complex8 + offset;
	    sPtr < sStopPtr; ++sPtr, ++cPtr )
      {
	*sPtr = (REAL4) cPtr->re;
      }
    break;
    case complex_16:
      sStopPtr = output + length;
      for ( sPtr = output, zPtr = input->data.complex16 + offset;
	    sPtr < sStopPtr; ++sPtr, ++zPtr )
      {
	*sPtr = (REAL4) zPtr->re;
      }
    default:
      ABORT (status, 
	     STOCHASTICCONDITIONDATAH_ETYPE, 
	     STOCHASTICCONDITIONDATAH_MSGETYPE);
  }
}

static void CastToComplex8 ( LALStatus *status, COMPLEX8 *output,
			     const multiDimData *input,
			     const UINT4 offset, const UINT4 length )
{
  REAL4     *sPtr;
  REAL8     *dPtr;
  INT2      *i2Ptr;
  UINT2     *u2Ptr;
  INT4      *i4Ptr;
  UINT4     *u4Ptr;
  INT8      *i8Ptr;
  UINT8     *u8Ptr;
  COMPLEX8  *cPtr, *cStopPtr;
  COMPLEX16 *zPtr;

  /* printf ("%d %d\n", input->type, complex_8); */
  switch (input->type)
  {
    case complex_8:
      cPtr = 
	memcpy ( output, input->data.complex8 + offset,
		 length * sizeof(COMPLEX8) );
      if (cPtr != output)
      {
        ABORT (status, 
               STOCHASTICCONDITIONDATAH_EALOC, 
               STOCHASTICCONDITIONDATAH_MSGEALOC);
      }
    break;
    case complex_16:
      cStopPtr = output + length;
      for ( cPtr = output, zPtr = input->data.complex16 + offset;
	    cPtr < cStopPtr; ++cPtr, ++zPtr )
      {
	cPtr->re = (REAL4) zPtr->re;
	cPtr->im = (REAL4) zPtr->im;
      }
    break;
    case real_4:
      cStopPtr = output + length;
      for ( cPtr = output, sPtr = input->data.real4 + offset;
	    cPtr < cStopPtr; ++cPtr, ++dPtr )
      {
	cPtr->re = *sPtr;
	cPtr->im = 0.0;
      }
    break;
    case real_8:
      cStopPtr = output + length;
      for ( cPtr = output, dPtr = input->data.real8 + offset;
	    cPtr < cStopPtr; ++cPtr, ++dPtr )
      {
	cPtr->re = (REAL4) *dPtr;
	cPtr->im = 0.0;
      }
    break;
    case int_2s:
      cStopPtr = output + length;
      for ( cPtr = output, i2Ptr = input->data.int2s + offset;
	    cPtr < cStopPtr; ++cPtr, ++i2Ptr )
      {
	cPtr->re = (REAL4) *i2Ptr;
	cPtr->im = 0.0;
      }
    break;
    case int_2u:
      cStopPtr = output + length;
      for ( cPtr = output, u2Ptr = input->data.int2u + offset;
	    cPtr < cStopPtr; ++cPtr, ++u2Ptr )
      {
	cPtr->re = (REAL4) *u2Ptr;
	cPtr->im = 0.0;
      }
    break;
    case int_4u:
      cStopPtr = output + length;
      for ( cPtr = output, u4Ptr = input->data.int4u + offset;
	    cPtr < cStopPtr; ++cPtr, ++u4Ptr )
      {
	cPtr->re = (REAL4) *u4Ptr;
	cPtr->im = 0.0;
      }
    break;
    case int_4s:
      cStopPtr = output + length;
      for ( cPtr = output, i4Ptr = input->data.int4s + offset;
	    cPtr < cStopPtr; ++cPtr, ++i4Ptr )
      {
	cPtr->re = (REAL4) *i4Ptr;
	cPtr->im = 0.0;
      }
    break;
    case int_8u:
      cStopPtr = output + length;
      for ( cPtr = output, u8Ptr = input->data.int8u + offset;
	    cPtr < cStopPtr; ++cPtr, ++u8Ptr )
      {
	cPtr->re = (REAL4) *u8Ptr;
	cPtr->im = 0.0;
      }
    break;
    case int_8s:
      cStopPtr = output + length;
      for ( cPtr = output, i8Ptr = input->data.int8s + offset;
	    cPtr < cStopPtr; ++cPtr, ++i8Ptr )
      {
	cPtr->re = (REAL4) *i8Ptr;
	cPtr->im = 0.0;
      }
    break;
    default:
      ABORT (status, 
	     STOCHASTICCONDITIONDATAH_ETYPE, 
	     STOCHASTICCONDITIONDATAH_MSGETYPE);
  }
}

static inline INT4 ctoi (CHAR c)
{
  switch(c)
  {
  case '0':
    return 0;
  case '1':
    return 1;
  case '2':
    return 2;
  case '3':
    return 3;
  case '4':
    return 4;
  case '5':
    return 5;
  case '6':
    return 6;
  case '7':
    return 7;
  case '8':
    return 8;
  case '9':
    return 9;
  default:
    return -1;
  }
}

#ifdef STOCHASTIC_SO_DEBUG
static void
PrintLALDetector(LALDetector *detector)
{
  printf( "  = { { %.15e, %.15e, %.15e },\n", detector->location[0],
          detector->location[1], detector->location[2] );
  printf( "      { { %.15g, %.15g, %.15g },\n", detector->response[0][0],
          detector->response[0][1], detector->response[0][2]);
  printf( "        { %.15g, %.15g, %.15g },\n", detector->response[1][0],
          detector->response[1][1], detector->response[1][2]);
  printf( "        { %.15g, %.15g, %.15g } },\n", detector->response[2][0],
          detector->response[2][1], detector->response[2][2]);
  printf("      { \"%s\",\n", detector->frDetector.name);
  printf("        %.15g, %.15g, %.15g,\n",
         detector->frDetector.vertexLongitudeRadians,
         detector->frDetector.vertexLatitudeRadians,
         detector->frDetector.vertexElevation);
  printf("        %.15e, %.15g,\n",
         detector->frDetector.xArmAltitudeRadians,
         detector->frDetector.xArmAzimuthRadians);
  printf("        %.15e, %.15g } }\n",
         detector->frDetector.yArmAltitudeRadians,
         detector->frDetector.yArmAzimuthRadians);
  return;
}
#endif
